@@ -23,23 +23,30 @@
  * License for more details.
  */
 
-#define PY_SSIZE_T_CLEAN
-#include <Python.h>
-#include <structmember.h>
-#include <stringobject.h>
-#include <mxDateTime.h>
-#include <string.h>
-
 #define PSYCOPG_MODULE
-#include "psycopg/config.h"
-#include "psycopg/python.h"
 #include "psycopg/psycopg.h"
+
 #include "psycopg/adapter_mxdatetime.h"
 #include "psycopg/microprotocols_proto.h"
 
-/* the pointer to the mxDateTime API is initialized by the module init code,
-   we just need to grab it */
-extern HIDDEN mxDateTimeModule_APIObject *mxDateTimeP;
+#include <mxDateTime.h>
+#include <string.h>
+
+
+/* Return 0 on success, -1 on failure, but don't set an exception */
+
+int
+psyco_adapter_mxdatetime_init(void)
+{
+    Dprintf("psyco_adapter_mxdatetime_init: mx.DateTime init");
+
+    if (mxDateTime_ImportModuleAndAPI()) {
+        Dprintf("psyco_adapter_mxdatetime_init: mx.DateTime initialization failed");
+        PyErr_Clear();
+        return -1;
+    }
+    return 0;
+}
 
 
 /* mxdatetime_str, mxdatetime_getquoted - return result of quoting */
@@ -106,7 +113,6 @@ mxdatetime_str(mxdatetimeObject *self)
 static PyObject *
 mxdatetime_getquoted(mxdatetimeObject *self, PyObject *args)
 {
-    if (!PyArg_ParseTuple(args, "")) return NULL;
     return mxdatetime_str(self);
 }
 
@@ -131,15 +137,15 @@ mxdatetime_conform(mxdatetimeObject *self, PyObject *args)
 /* object member list */
 
 static struct PyMemberDef mxdatetimeObject_members[] = {
-    {"adapted", T_OBJECT, offsetof(mxdatetimeObject, wrapped), RO},
-    {"type", T_INT, offsetof(mxdatetimeObject, type), RO},
+    {"adapted", T_OBJECT, offsetof(mxdatetimeObject, wrapped), READONLY},
+    {"type", T_INT, offsetof(mxdatetimeObject, type), READONLY},
     {NULL}
 };
 
 /* object method table */
 
 static PyMethodDef mxdatetimeObject_methods[] = {
-    {"getquoted", (PyCFunction)mxdatetime_getquoted, METH_VARARGS,
+    {"getquoted", (PyCFunction)mxdatetime_getquoted, METH_NOARGS,
      "getquoted() -> wrapped object value as SQL date/time"},
     {"__conform__", (PyCFunction)mxdatetime_conform, METH_VARARGS, NULL},
     {NULL}  /* Sentinel */
@@ -152,7 +158,7 @@ mxdatetime_setup(mxdatetimeObject *self, PyObject *obj, int type)
 {
     Dprintf("mxdatetime_setup: init mxdatetime object at %p, refcnt = "
         FORMAT_CODE_PY_SSIZE_T,
-        self, ((PyObject *)self)->ob_refcnt
+        self, Py_REFCNT(self)
       );
 
     self->type = type;
@@ -161,20 +167,11 @@ mxdatetime_setup(mxdatetimeObject *self, PyObject *obj, int type)
 
     Dprintf("mxdatetime_setup: good mxdatetime object at %p, refcnt = "
         FORMAT_CODE_PY_SSIZE_T,
-        self, ((PyObject *)self)->ob_refcnt
+        self, Py_REFCNT(self)
       );
     return 0;
 }
 
-static int
-mxdatetime_traverse(PyObject *obj, visitproc visit, void *arg)
-{
-    mxdatetimeObject *self = (mxdatetimeObject *)obj;
-
-    Py_VISIT(self->wrapped);
-    return 0;
-}
-
 static void
 mxdatetime_dealloc(PyObject* obj)
 {
@@ -184,10 +181,10 @@ mxdatetime_dealloc(PyObject* obj)
 
     Dprintf("mxdatetime_dealloc: deleted mxdatetime object at %p, refcnt = "
         FORMAT_CODE_PY_SSIZE_T,
-        obj, obj->ob_refcnt
+        obj, Py_REFCNT(obj)
       );
 
-    obj->ob_type->tp_free(obj);
+    Py_TYPE(obj)->tp_free(obj);
 }
 
 static int
@@ -208,12 +205,6 @@ mxdatetime_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
     return type->tp_alloc(type, 0);
 }
 
-static void
-mxdatetime_del(PyObject* self)
-{
-    PyObject_GC_Del(self);
-}
-
 static PyObject *
 mxdatetime_repr(mxdatetimeObject *self)
 {
@@ -227,64 +218,43 @@ mxdatetime_repr(mxdatetimeObject *self)
 "MxDateTime(mx, type) -> new mx.DateTime wrapper object"
 
 PyTypeObject mxdatetimeType = {
-    PyObject_HEAD_INIT(NULL)
-    0,
+    PyVarObject_HEAD_INIT(NULL, 0)
     "psycopg2._psycopg.MxDateTime",
-    sizeof(mxdatetimeObject),
-    0,
+    sizeof(mxdatetimeObject), 0,
     mxdatetime_dealloc, /*tp_dealloc*/
     0,          /*tp_print*/
     0,          /*tp_getattr*/
     0,          /*tp_setattr*/
-
     0,          /*tp_compare*/
     (reprfunc)mxdatetime_repr, /*tp_repr*/
     0,          /*tp_as_number*/
     0,          /*tp_as_sequence*/
     0,          /*tp_as_mapping*/
     0,          /*tp_hash */
-
     0,          /*tp_call*/
     (reprfunc)mxdatetime_str, /*tp_str*/
     0,          /*tp_getattro*/
     0,          /*tp_setattro*/
     0,          /*tp_as_buffer*/
-
-    Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
-
+    Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE, /*tp_flags*/
     mxdatetimeType_doc, /*tp_doc*/
-
-    mxdatetime_traverse, /*tp_traverse*/
+    0,          /*tp_traverse*/
     0,          /*tp_clear*/
-
     0,          /*tp_richcompare*/
     0,          /*tp_weaklistoffset*/
-
     0,          /*tp_iter*/
     0,          /*tp_iternext*/
-
-    /* Attribute descriptor and subclassing stuff */
-
     mxdatetimeObject_methods, /*tp_methods*/
     mxdatetimeObject_members, /*tp_members*/
     0,          /*tp_getset*/
     0,          /*tp_base*/
     0,          /*tp_dict*/
-
     0,          /*tp_descr_get*/
     0,          /*tp_descr_set*/
     0,          /*tp_dictoffset*/
-
     mxdatetime_init, /*tp_init*/
     0,          /*tp_alloc*/
     mxdatetime_new, /*tp_new*/
-    (freefunc)mxdatetime_del, /*tp_free  Low-level free-memory routine */
-    0,          /*tp_is_gc For PyObject_IS_GC */
-    0,          /*tp_bases*/
-    0,          /*tp_mro method resolution order */
-    0,          /*tp_cache*/
-    0,          /*tp_subclasses*/
-    0           /*tp_weaklist*/
 };
 
 
@@ -301,7 +271,7 @@ psyco_Date(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "iii", &year, &month, &day))
         return NULL;
 
-    mx = mxDateTimeP->DateTime_FromDateAndTime(year, month, day, 0, 0, 0.0);
+    mx = mxDateTime.DateTime_FromDateAndTime(year, month, day, 0, 0, 0.0);
     if (mx == NULL) return NULL;
 
     res = PyObject_CallFunction((PyObject *)&mxdatetimeType, "Oi", mx,
@@ -320,7 +290,7 @@ psyco_Time(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "iid", &hours, &minutes, &seconds))
         return NULL;
 
-    mx = mxDateTimeP->DateTimeDelta_FromTime(hours, minutes, seconds);
+    mx = mxDateTime.DateTimeDelta_FromTime(hours, minutes, seconds);
     if (mx == NULL) return NULL;
 
     res = PyObject_CallFunction((PyObject *)&mxdatetimeType, "Oi", mx,
@@ -341,7 +311,7 @@ psyco_Timestamp(PyObject *self, PyObject *args)
                           &hour, &minute, &second))
         return NULL;
 
-    mx = mxDateTimeP->DateTime_FromDateAndTime(year, month, day,
+    mx = mxDateTime.DateTime_FromDateAndTime(year, month, day,
                                                hour, minute, second);
     if (mx == NULL) return NULL;
 
@@ -360,7 +330,7 @@ psyco_DateFromTicks(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args,"d", &ticks))
         return NULL;
 
-    if (!(mx = mxDateTimeP->DateTime_FromTicks(ticks)))
+    if (!(mx = mxDateTime.DateTime_FromTicks(ticks)))
         return NULL;
 
     res = PyObject_CallFunction((PyObject *)&mxdatetimeType, "Oi", mx,
@@ -378,10 +348,10 @@ psyco_TimeFromTicks(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args,"d", &ticks))
         return NULL;
 
-    if (!(dt = mxDateTimeP->DateTime_FromTicks(ticks)))
+    if (!(dt = mxDateTime.DateTime_FromTicks(ticks)))
         return NULL;
 
-    if (!(mx = mxDateTimeP->DateTimeDelta_FromDaysAndSeconds(
+    if (!(mx = mxDateTime.DateTimeDelta_FromDaysAndSeconds(
             0, ((mxDateTimeObject*)dt)->abstime)))
     {
         Py_DECREF(dt);
@@ -404,7 +374,7 @@ psyco_TimestampFromTicks(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "d", &ticks))
         return NULL;
 
-    if (!(mx = mxDateTimeP->DateTime_FromTicks(ticks)))
+    if (!(mx = mxDateTime.DateTime_FromTicks(ticks)))
         return NULL;
 
     res = PyObject_CallFunction((PyObject *)&mxdatetimeType, "Oi", mx,
@@ -420,7 +390,7 @@ psyco_DateFromMx(PyObject *self, PyObject *args)
 {
     PyObject *mx;
 
-    if (!PyArg_ParseTuple(args, "O!", mxDateTimeP->DateTime_Type, &mx))
+    if (!PyArg_ParseTuple(args, "O!", mxDateTime.DateTime_Type, &mx))
         return NULL;
 
     return PyObject_CallFunction((PyObject *)&mxdatetimeType, "Oi", mx,
@@ -432,7 +402,7 @@ psyco_TimeFromMx(PyObject *self, PyObject *args)
 {
     PyObject *mx;
 
-    if (!PyArg_ParseTuple(args, "O!", mxDateTimeP->DateTimeDelta_Type, &mx))
+    if (!PyArg_ParseTuple(args, "O!", mxDateTime.DateTimeDelta_Type, &mx))
         return NULL;
 
     return PyObject_CallFunction((PyObject *)&mxdatetimeType, "Oi", mx,
@@ -444,7 +414,7 @@ psyco_TimestampFromMx(PyObject *self, PyObject *args)
 {
     PyObject *mx;
 
-    if (!PyArg_ParseTuple(args, "O!", mxDateTimeP->DateTime_Type, &mx))
+    if (!PyArg_ParseTuple(args, "O!", mxDateTime.DateTime_Type, &mx))
         return NULL;
 
     return PyObject_CallFunction((PyObject *)&mxdatetimeType, "Oi", mx,
@@ -456,7 +426,7 @@ psyco_IntervalFromMx(PyObject *self, PyObject *args)
 {
     PyObject *mx;
 
-    if (!PyArg_ParseTuple(args, "O!", mxDateTimeP->DateTime_Type, &mx))
+    if (!PyArg_ParseTuple(args, "O!", mxDateTime.DateTime_Type, &mx))
         return NULL;
 
     return PyObject_CallFunction((PyObject *)&mxdatetimeType, "Oi", mx,
